from typing import List
from graphviz import Digraph
from utils.csv_reader import group_logs_by_users_from
from sessionizator.strategies.session_path import SessionPath
from sessionizator.strategies.time_referer_based_sessionizator import get_sessions_with_time_referer_heuristic
from sessionizator.strategies.time_based_sessionizator import get_sessions_with_time_heuristic
from utils.constants import SessionizationHeuristic


# It is important to have graphviz installed in python environment
# as well as an executable iin the OS.
def build_graph(csv_source: str, folder: str, strategy = SessionizationHeuristic.TIME_REFERER_BASED) -> None:
  logs_by_user = group_logs_by_users_from(csv_source)

  #there's only going to be one
  for _, logs in logs_by_user.items():
    sessions = None
    if strategy == SessionizationHeuristic.TIME_BASED:
      sessions = get_sessions_with_time_heuristic(logs)
    elif strategy == SessionizationHeuristic.TIME_REFERER_BASED:
      sessions = get_sessions_with_time_referer_heuristic(logs)

    build_graph_from_sessions(sessions, folder)

def build_graph_from_sessions(sessions: List[SessionPath], folder: str, user = '') -> None:
  for session_id, session_path in enumerate(sessions):
    graph = Digraph()
    
    first_path = session_path
    graph.edge(first_path.log.referer_uri, first_path.log.uri, 'Edge  0')
    build_graph_from_session_path(None, first_path, graph, 1)

    name = folder + user + str(session_id)
    graph.render(name)

def build_graph_from_session_path(previous_path: SessionPath, current_path: SessionPath, graph: Digraph, depth: int) -> None:
  if not current_path:
    return

  if previous_path:
    graph.edge(previous_path.uri, current_path.uri, 'Edge ' + str(depth))
  
  for _, child in current_path.children.items():
    build_graph_from_session_path(current_path, child, graph, depth + 1)