from datetime import datetime

class Log:
  # Available fields
  # 'ip', 'time', 'request', 'status', 'size', 'referer', 'user_agent'
  def __init__(self, data_frame_row: dict):
    self.__ip = data_frame_row['ip']
    self.__request = data_frame_row['request']
    self.__referer = data_frame_row['referer']
    self.__status = data_frame_row['status']
    self.__size = data_frame_row['size']
    self.__user_agent = str(data_frame_row['user_agent'])

    parsed_time = datetime.strptime(data_frame_row['time'], r'%Y-%m-%d %H:%M:%S')
    self.__time = parsed_time.timestamp()

    self.__referer_url, self.__referer_uri = self.__get_referer_url_and_uri(self.__referer)

    self.__id = self.__ip + ' ' + self.__user_agent

    self.__method, self.__uri = self.__get_method_and_current_uri(self.__request)


  @property
  def id(self) -> str:
    return self.__id

  # It's a combination of the IP and the UserAgent of a log
  @property
  def ip(self) -> str:
    return self.__ip

  @property
  def request(self) -> str:
    return self.__request

  @property
  def method(self) -> str:
    return self.__method

  @property
  def uri(self) -> str:
    return self.__uri
  
  @property
  def status(self) -> str:
    return self.__status

  @property
  def size(self) -> str:
    return self.__size

  @property
  def referer(self) -> str:
    return self.__referer if self.__referer == self.__referer else None

  @property
  def referer_url(self) -> str:
    return self.__referer_url

  @property
  def referer_uri(self) -> str:
    return self.__referer_uri

  @property
  def user_agent(self) -> str:
    return self.__user_agent

  @property
  def time(self) -> float:
    return self.__time


  def __get_referer_url_and_uri(self, referer: str) -> (str, str):
    # referer != referer checks whether referrer is NaN
    if not referer or referer != referer:
      return '-', '-'

    slashes_seen = 0
    url = []
    uri = []
    for letter in referer:
      if letter == '/':
        slashes_seen += 1
      if slashes_seen >= 3:
        uri.append(letter)
        continue

      url.append(letter)

    return ''.join(url), ''.join(uri)

  def __get_method_and_current_uri(self, request: str) -> (str, str):
    method = []
    i = 0
    while i < len(request):
      if request[i] == ' ':
        break

      method.append(request[i])
      i += 1

    i += 1
    uri = []
    while i < len(request):
      if request[i] == ' ':
        break

      uri.append(request[i])
      i += 1

    return ''.join(method), ''.join(uri)


  def __repr__(self):
    return '"' + str(self.__time) + ' ' + str(self.__ip) + ' ' + str(self.__request) + ' ' + str(self.__referer) + '"'

  def __str__(self):
    return '"' + str(self.__time) + ' ' + str(self.__ip) + ' ' + str(self.__request) + ' ' + str(self.__referer) + '"'