from typing import List
from utils.csv_reader import group_logs_by_users_from
from utils.constants import SessionizationHeuristic
from sessionizator.strategies.session_path import SessionPath
from sessionizator.strategies.time_referer_based_sessionizator import get_sessions_with_time_referer_heuristic
from sessionizator.strategies.time_based_sessionizator import get_sessions_with_time_heuristic

def get_statistics(csv_source: str, strategy = SessionizationHeuristic.TIME_REFERER_BASED) -> (List[int], List[int], List[int]):
  logs_by_user = group_logs_by_users_from(csv_source)

  logs_per_user = []
  logs_per_session = []
  sessions_per_user = []
  print('Gathering data for statistics ...')
  for _, logs in logs_by_user.items():
    sessions = None
    if strategy == SessionizationHeuristic.TIME_BASED:
      sessions = get_sessions_with_time_heuristic(logs)
    elif strategy == SessionizationHeuristic.TIME_REFERER_BASED:
      sessions = get_sessions_with_time_referer_heuristic(logs)

    sessions_per_user.append(len(sessions))
    __get_data(sessions, logs_per_user, logs_per_session)

  print('Done')

  return sessions_per_user, logs_per_session, logs_per_user

def __get_data(sessions: List[SessionPath], logs_per_user: List[int], logs_per_session: List[int]) -> None:
  total_logs = 0

  for session in sessions:
    nodes_count = __count_nodes(session)
    total_logs += nodes_count
    logs_per_session.append(nodes_count)

  logs_per_user.append(total_logs)

def __count_nodes(session: SessionPath) -> int:
  if not session:
    return 0

  nodes = 1
  for _, child in session.children.items():
    nodes += __count_nodes(child)

  return nodes