from collections import defaultdict

from logs.log import Log
from typing import List, Dict, DefaultDict
from utils.constants import SessionizationHeuristic
from sessionizator.strategies.session_path import SessionPath
from sessionizator.strategies.time_based_sessionizator import get_sessions_with_time_heuristic
from sessionizator.strategies.time_referer_based_sessionizator import get_sessions_with_time_referer_heuristic


def get_sessions_by_user(logs_by_user: Dict[str, List[Log]], strategy = SessionizationHeuristic.TIME_BASED) -> DefaultDict[str, List[SessionPath]]:
  sessions_by_user = defaultdict(list)
  
  print('Getting sessions by user...')
  for user, logs in logs_by_user.items():
    sessions = None
    if strategy == SessionizationHeuristic.TIME_BASED:
      sessions = get_sessions_with_time_heuristic(logs)
    elif strategy == SessionizationHeuristic.TIME_REFERER_BASED:
      sessions = get_sessions_with_time_referer_heuristic(logs)

    sessions_by_user[user] = sessions

  print('Finished getting sessions')

  return sessions_by_user