from logs.log import Log
from my_types import Session
from typing import List, Dict, Set

class SessionPath:
  def __init__(self, log: Log, previous_session_path = None):
    self.__parent_branch = previous_session_path
    self.log = log
    self.children = {}
    self.__hash_id = self.uri + str(self.last_log_time)
    self.__super_parent = self.__parent_branch.__super_parent if self.__parent_branch else self

    if self.__parent_branch:
      self.__parent_branch.__add_child(self)

  @property
  def last_log_time(self) -> float:
    return self.log.time

  @property
  def uri(self) -> str:
    return self.log.uri

  @property
  def id(self) -> str:
    return self.log.id

  @property
  def hash_id(self) -> str:
    return self.__hash_id

  @property
  def super_parent(self) -> 'SessionPath':
    return self.__super_parent

  def to_session(self, graph: Dict[str, 'SessionPath'], seen: set) -> 'SessionPath':    
    to_return = self.super_parent
    current_node = self
    while current_node.__parent_branch:
      if not current_node.children: 
        # If I have no children, then I'm not part of any other session
        # path so I can delete myself from the graph
        if current_node.__parent_branch:
          # since I'm deleting myself from the graph
          # I should also do so from my parent's children
          # I can do that bacause I'm not part of any other session path
          self.__remove_from_parents_children()

        del graph[current_node.uri]

      current_node = current_node.__parent_branch

    if to_return.hash_id in seen:
      return None

    seen.add(to_return.hash_id)
    
    return to_return

  def __add_child(self, child: 'SessionPath'):
    self.children[child.__hash_id] = child

  def __remove_from_parents_children(self) -> bool:
    parent = self.__parent_branch
    if self in parent.children:
      parent.children.pop(self.__hash_id, None)
      return True

    return False

  def __hash__(self):
    return self.id.__hash__()

  def __eq__(self, other_path):
    return self.id == other_path.id