import pandas

from collections import defaultdict
from typing import DefaultDict

from my_types import PandasDataFrame
from logs.log import Log

# This only processes CSVs containing filtered data from the
# filter_data.ipynb notebook
def group_logs_by_users_from(csv_to_read: str) -> DefaultDict[str, list]:
  if not csv_to_read:
    raise Exception(message='Error: no csv file specified!')
  
  data_chunks = __read_csv(csv_to_read) 

  return __group_logs_by_user(data_chunks) 

def __read_csv(csv_to_read: str) -> PandasDataFrame:
  print('Will load csv from: ', csv_to_read)
  data_chunks = pandas.read_csv(csv_to_read,
    names=['ip', 'time', 'request', 'status', 'size', 'referer', 'user_agent'],
    converters={ 'ip' : str }
    )

  return data_chunks

def __group_logs_by_user(data_chunks: PandasDataFrame) -> DefaultDict[str, list]:
  logs_by_id = defaultdict(list)

  print('Grouping logs by id...')
  # We start frouser1 to avoid the column names row from thusercsv
  for i in range(1, len(data_chunks)):
    row = data_chunks.iloc[i]
    
    log = Log(row)
    logs_by_id[log.id].append(log)

  print('Done grouping logs by id')

  return logs_by_id